@@ -6,20 +6,8 @@
  *         Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *                               <zeeshan.ali@nokia.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -98,7 +86,7 @@ G_DEFINE_TYPE_EXTENDED (GUPnPContext,
                         G_ADD_PRIVATE(GUPnPContext)
                         G_IMPLEMENT_INTERFACE
                                 (G_TYPE_INITABLE,
-                                 gupnp_context_initable_iface_init));
+                                 gupnp_context_initable_iface_init))
 
 enum {
         PROP_0,
@@ -499,7 +487,7 @@ gupnp_context_class_init (GUPnPContextClass *klass)
          * sends Accept-Language and no language-specific pages to serve
          * exist. The property defaults to 'en'.
          *
-         * Since: 0.17.0
+         * Since: 0.18.0
          **/
         g_object_class_install_property
                 (object_class,
@@ -664,10 +652,10 @@ _gupnp_context_get_server_uri (GUPnPContext *context)
 
 /**
  * gupnp_context_new:
- * @iface: (allow-none): The network interface to use, or %NULL to
+ * @iface: (nullable): The network interface to use, or %NULL to
  * auto-detect.
  * @port: Port to run on, or 0 if you don't care what port is used.
- * @error: A location to store a #GError, or %NULL
+ * @error: (inout)(optional)(nullable): A location to store a #GError, or %NULL
  *
  * Create a new #GUPnPContext with the specified @main_context, @iface and
  * @port.
@@ -771,14 +759,14 @@ host_path_data_set_language (HostPathData *data, const char *language)
  * @context: A #GUPnPContext
  * @language: A language tag as defined in RFC 2616 3.10
  *
- * Set the default language for the Content-Length header to @language.
+ * Set the default language for the Content-Language header to @language.
  *
  * If the client sends an Accept-Language header the UPnP HTTP server
  * is required to send a Content-Language header in return. If there are
  * no files hosted in languages which match the requested ones the
  * Content-Language header is set to this value. The default value is "en".
  *
- * Since: 0.17.0
+ * Since: 0.18.0
  */
 void
 gupnp_context_set_default_language (GUPnPContext *context,
@@ -815,7 +803,7 @@ gupnp_context_set_default_language (GUPnPContext *context,
  * Returns: (transfer none): The default content of the Content-Language
  * header.
  *
- * Since: 0.17.0
+ * Since: 0.18.0
  */
 const char *
 gupnp_context_get_default_language (GUPnPContext *context)
@@ -1308,7 +1296,7 @@ path_compare_func (HostPathData *path_data,
  *
  * Return value: %TRUE on success, %FALSE otherwise.
  *
- * Since: 0.13.3
+ * Since: 0.14.0
  **/
 gboolean
 gupnp_context_host_path_for_agent (GUPnPContext *context,
@@ -1405,7 +1393,7 @@ gupnp_context_get_acl (GUPnPContext *context)
 /**
  * gupnp_context_set_acl:
  * @context: A #GUPnPContext
- * @acl: (allow-none): The new access control list or %NULL to remove the
+ * @acl: (nullable): The new access control list or %NULL to remove the
  * current list.
  *
  * Attach or remove the assoicated access control list to this context. If
@@ -1531,7 +1519,7 @@ gupnp_acl_server_handler (SoupServer *server,
  * @path: the toplevel path for the handler.
  * @callback: callback to invoke for requests under @path
  * @user_data: the user_data passed to @callback
- * @destroy: (allow-none): A #GDestroyNotify for @user_data or %NULL if none.
+ * @destroy: (nullable): A #GDestroyNotify for @user_data or %NULL if none.
  *
  * Add a #SoupServerCallback to the #GUPnPContext<!-- -->'s #SoupServer.
  *
@@ -1614,7 +1602,7 @@ gupnp_context_remove_server_handler (GUPnPContext *context, const char *path)
  * Returns: A re-written version of the @uri if the context is on a link-local
  * IPv6 address, a copy of the @uri otherwise or %NULL if @uri was invalid
  *
- * Since: 1.1.1
+ * Since: 1.2.0
  */
 char *
 gupnp_context_rewrite_uri (GUPnPContext *context, const char *uri)
@@ -1684,32 +1672,6 @@ gupnp_context_rewrite_uri_to_uri (GUPnPContext *context, const char *uri)
         return soup_uri;
 }
 
-gboolean
-gupnp_context_ip_is_ours (GUPnPContext *context, const char *address)
-{
-        // TODO: Could easily be in GSSDPClient, which does something similar
-        gboolean retval = FALSE;
-        GInetAddress *addr = NULL;
-        GInetAddressMask *mask = NULL;
-
-        addr = g_inet_address_new_from_string (address);
-
-        // Link-local addresses are reachable
-        if (g_inet_address_get_is_link_local (addr)) {
-                retval = TRUE;
-                goto out;
-        }
-
-        mask = gssdp_client_get_address_mask (GSSDP_CLIENT (context));
-        retval = g_inet_address_mask_matches (mask, addr);
-        g_object_unref (mask);
-
-out:
-        g_object_unref (addr);
-
-        return retval;
-}
-
 gboolean
 validate_host_header (const char *host_header,
                       const char *host_ip,
