@@ -3,20 +3,8 @@
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -53,7 +41,7 @@ typedef struct _GUPnPDeviceInfoPrivate GUPnPDeviceInfoPrivate;
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GUPnPDeviceInfo,
                                      gupnp_device_info,
-                                     G_TYPE_OBJECT);
+                                     G_TYPE_OBJECT)
 
 
 enum {
@@ -168,20 +156,10 @@ gupnp_device_info_dispose (GObject *object)
         info = GUPNP_DEVICE_INFO (object);
         priv = gupnp_device_info_get_instance_private (info);
 
-        if (priv->factory) {
-                g_object_unref (priv->factory);
-                priv->factory = NULL;
-        }
-
-        if (priv->context) {
-                g_object_unref (priv->context);
-                priv->context = NULL;
-        }
-
-        if (priv->doc) {
-                g_object_unref (priv->doc);
-                priv->doc = NULL;
-        }
+        g_clear_object (&priv->factory);
+        g_clear_object (&priv->factory);
+        g_clear_object (&priv->context);
+        g_clear_object (&priv->doc);
 
         G_OBJECT_CLASS (gupnp_device_info_parent_class)->dispose (object);
 }
@@ -524,7 +502,7 @@ gupnp_device_info_get_friendly_name (GUPnPDeviceInfo *info)
  *
  * Get the manufacturer of the device.
  *
- * Return value: A string, or %NULL. g_free() after use.
+ * Return value:(nullable)(transfer full): A string, or %NULL. g_free() after use.
  **/
 char *
 gupnp_device_info_get_manufacturer (GUPnPDeviceInfo *info)
@@ -543,9 +521,9 @@ gupnp_device_info_get_manufacturer (GUPnPDeviceInfo *info)
  * gupnp_device_info_get_manufacturer_url:
  * @info: A #GUPnPDeviceInfo
  *
- * Get a URL pointing to the manufacturer's website.
+ * Get an URL pointing to the manufacturer's website.
  *
- * Return value: A string, or %NULL. g_free() after use.
+ * Return value:(nullable)(transfer full): A string, or %NULL. g_free() after use.
  **/
 char *
 gupnp_device_info_get_manufacturer_url (GUPnPDeviceInfo *info)
@@ -567,7 +545,7 @@ gupnp_device_info_get_manufacturer_url (GUPnPDeviceInfo *info)
  *
  * Get the description of the device model.
  *
- * Return value: A string, or %NULL. g_free() after use.
+ * Return value:(nullable)(transfer full): A string, or %NULL. g_free() after use.
  **/
 char *
 gupnp_device_info_get_model_description (GUPnPDeviceInfo *info)
@@ -588,7 +566,7 @@ gupnp_device_info_get_model_description (GUPnPDeviceInfo *info)
  *
  * Get the model name of the device.
  *
- * Return value: A string, or %NULL. g_free() after use.
+ * Return value:(nullable)(transfer full): A string, or %NULL. g_free() after use.
  **/
 char *
 gupnp_device_info_get_model_name (GUPnPDeviceInfo *info)
@@ -609,7 +587,7 @@ gupnp_device_info_get_model_name (GUPnPDeviceInfo *info)
  *
  * Get the model number of the device.
  *
- * Return value: A string, or %NULL. g_free() after use.
+ * Return value:(nullable)(transfer full): A string, or %NULL. g_free() after use.
  **/
 char *
 gupnp_device_info_get_model_number (GUPnPDeviceInfo *info)
@@ -628,9 +606,9 @@ gupnp_device_info_get_model_number (GUPnPDeviceInfo *info)
  * gupnp_device_info_get_model_url:
  * @info: A #GUPnPDeviceInfo
  *
- * Get a URL pointing to the device model's website.
+ * Get an URL pointing to the device model's website.
  *
- * Return value: A string, or %NULL. g_free() after use.
+ * Return value:(nullable)(transfer full): A string, or %NULL. g_free() after use.
  **/
 char *
 gupnp_device_info_get_model_url (GUPnPDeviceInfo *info)
@@ -652,7 +630,7 @@ gupnp_device_info_get_model_url (GUPnPDeviceInfo *info)
  *
  * Get the serial number of the device.
  *
- * Return value: A string, or %NULL. g_free() after use.
+ * Return value:(nullable)(transfer full): A string, or %NULL. g_free() after use.
  **/
 char *
 gupnp_device_info_get_serial_number (GUPnPDeviceInfo *info)
@@ -673,7 +651,7 @@ gupnp_device_info_get_serial_number (GUPnPDeviceInfo *info)
  *
  * Get the Universal Product Code of the device.
  *
- * Return value: A string, or %NULL. g_free() after use.
+ * Return value:(nullable)(transfer full): A string, or %NULL. g_free() after use.
  **/
 char *
 gupnp_device_info_get_upc (GUPnPDeviceInfo *info)
@@ -692,10 +670,10 @@ gupnp_device_info_get_upc (GUPnPDeviceInfo *info)
  * gupnp_device_info_get_presentation_url:
  * @info: A #GUPnPDeviceInfo
  *
- * Get a URL pointing to the device's presentation page, for web-based
+ * Get an URL pointing to the device's presentation page, for web-based
  * administration.
  *
- * Return value: A string, or %NULL. g_free() after use.
+ * Return value:(nullable)(transfer full): A string, or %NULL. g_free() after use.
  **/
 char *
 gupnp_device_info_get_presentation_url (GUPnPDeviceInfo *info)
@@ -722,7 +700,7 @@ typedef struct {
 } Icon;
 
 static Icon *
-icon_parse (G_GNUC_UNUSED GUPnPDeviceInfo *info, xmlNode *element)
+icon_parse (xmlNode *element)
 {
         Icon *icon;
 
@@ -757,24 +735,24 @@ icon_free (Icon *icon)
 /**
  * gupnp_device_info_get_icon_url:
  * @info: A #GUPnPDeviceInfo
- * @requested_mime_type: (allow-none) (transfer none): The requested file
+ * @requested_mime_type: (nullable) (transfer none): The requested file
  * format, or %NULL for any
  * @requested_depth: The requested color depth, or -1 for any
  * @requested_width: The requested width, or -1 for any
  * @requested_height: The requested height, or -1 for any
  * @prefer_bigger: %TRUE if a bigger, rather than a smaller icon should be
  * returned if no exact match could be found
- * @mime_type: (out) (allow-none): The location where to store the the format
+ * @mime_type: (out) (optional): The location where to store the the format
  * of the returned icon, or %NULL. The returned string should be freed after
  * use
- * @depth: (out) (allow-none):  The location where to store the depth of the
+ * @depth: (out) (optional):  The location where to store the depth of the
  * returned icon, or %NULL
- * @width: (out) (allow-none): The location where to store the width of the
+ * @width: (out) (optional): The location where to store the width of the
  * returned icon, or %NULL
- * @height: (out) (allow-none): The location where to store the height of the
+ * @height: (out) (optional): The location where to store the height of the
  * returned icon, or %NULL
  *
- * Get a URL pointing to the icon most closely matching the
+ * Get an URL pointing to the icon most closely matching the
  * given criteria, or %NULL. If @requested_mime_type is set, only icons with
  * this mime type will be returned. If @requested_depth is set, only icons with
  * this or lower depth will be returned. If @requested_width and/or
@@ -782,7 +760,7 @@ icon_free (Icon *icon)
  * returned, unless @prefer_bigger is set, in which case the next biggest icon
  * will be returned. The returned strings should be freed.
  *
- * Return value: (transfer full): a string, or %NULL.  g_free() after use.
+ * Return value:(nullable)(transfer full): a string, or %NULL.  g_free() after use.
  **/
 char *
 gupnp_device_info_get_icon_url (GUPnPDeviceInfo *info,
@@ -819,7 +797,7 @@ gupnp_device_info_get_icon_url (GUPnPDeviceInfo *info,
                 if (!strcmp ("icon", (char *) element->name)) {
                         gboolean mime_type_ok;
 
-                        icon = icon_parse (info, element);
+                        icon = icon_parse (element);
 
                         if (requested_mime_type) {
                                 if (icon->mime_type)
@@ -1022,7 +1000,7 @@ resource_type_match (const char *query,
  * Get a #GList of strings that represent the device class and version as
  * announced in the device description file using the &lt;dlna:X_DLNADOC&gt;
  * element.
- * Returns: (transfer full) (element-type utf8): a #GList of newly allocated strings or
+ * Returns:(nullable)(transfer full) (element-type utf8): a #GList of newly allocated strings or
  * %NULL if the device description doesn't contain the &lt;dlna:X_DLNADOC&gt;
  * element.
  *
@@ -1069,11 +1047,11 @@ gupnp_device_info_list_dlna_device_class_identifier (GUPnPDeviceInfo *info)
  * Get a #GList of strings that represent the device capabilities as announced
  * in the device description file using the &lt;dlna:X_DLNACAP&gt; element.
  *
- * Returns: (transfer full) (element-type utf8): a #GList of newly allocated strings or
+ * Returns:(nullable)(transfer full) (element-type utf8): a #GList of newly allocated strings or
  * %NULL if the device description doesn't contain the &lt;dlna:X_DLNACAP&gt;
  * element.
  *
- * Since: 0.13.0
+ * Since: 0.14.0
  **/
 GList *
 gupnp_device_info_list_dlna_capabilities (GUPnPDeviceInfo *info)
@@ -1129,10 +1107,10 @@ gupnp_device_info_list_dlna_capabilities (GUPnPDeviceInfo *info)
  * This function provides generic access to the contents of arbitrary elements
  * in the device description file.
  *
- * Return value: a newly allocated string or %NULL if the device
+ * Return value:(nullable)(transfer full): a newly allocated string or %NULL if the device
  *               description doesn't contain the given @element
  *
- * Since: 0.13.0
+ * Since: 0.14.0
  **/
 char *
 gupnp_device_info_get_description_value (GUPnPDeviceInfo *info,
@@ -1162,7 +1140,7 @@ gupnp_device_info_get_description_value (GUPnPDeviceInfo *info,
  * must cache any used devices if it wishes to keep them around and re-use
  * them.
  *
- * Return value: (element-type GUPnP.DeviceInfo) (transfer full): a #GList of
+ * Return value:(nullable)(element-type GUPnP.DeviceInfo) (transfer full): a #GList of
  * new #GUPnPDeviceInfo objects.
  **/
 GList *
@@ -1208,7 +1186,7 @@ gupnp_device_info_list_devices (GUPnPDeviceInfo *info)
  * Get a #GList of strings representing the types of the devices
  * directly contained in @info.
  *
- * Return value: (element-type utf8) (transfer full): A #GList of strings. The
+ * Return value:(nullable)(element-type utf8) (transfer full): A #GList of strings. The
  * elements should be g_free()'d and the list should be g_list_free()'d.
  **/
 GList *
@@ -1259,7 +1237,7 @@ gupnp_device_info_list_device_types (GUPnPDeviceInfo *info)
  * this function a new object is created. The application must cache any used
  * devices if it wishes to keep them around and re-use them.
  *
- * Returns: (transfer full)(allow-none): A new #GUPnPDeviceInfo.
+ * Returns: (transfer full)(nullable): A new #GUPnPDeviceInfo.
  **/
 GUPnPDeviceInfo *
 gupnp_device_info_get_device (GUPnPDeviceInfo *info,
@@ -1325,7 +1303,7 @@ gupnp_device_info_get_device (GUPnPDeviceInfo *info,
  * function new objects are created. The application must cache any used
  * services if it wishes to keep them around and re-use them.
  *
- * Return value: (element-type GUPnP.ServiceInfo) (transfer full) : A #GList of
+ * Return value: (nullable)(element-type GUPnP.ServiceInfo) (transfer full) : A #GList of
  * new #GUPnPServiceInfo objects.
  */
 GList *
@@ -1371,7 +1349,7 @@ gupnp_device_info_list_services (GUPnPDeviceInfo *info)
  * Get a #GList of strings representing the types of the services
  * directly contained in @info.
  *
- * Return value: (element-type utf8) (transfer full): A #GList of strings. The
+ * Return value: (nullable)(element-type utf8) (transfer full): A #GList of strings. The
  * elements should be g_free()'d and the list should be g_list_free()'d.
  **/
 GList *
@@ -1422,7 +1400,7 @@ gupnp_device_info_list_service_types (GUPnPDeviceInfo *info)
  * this function a new object is created. The application must cache any used
  * services if it wishes to keep them around and re-use them.
  *
- * Returns: (transfer full)(allow-none): A #GUPnPServiceInfo.
+ * Returns: (nullable)(transfer full): A #GUPnPServiceInfo.
  **/
 GUPnPServiceInfo *
 gupnp_device_info_get_service (GUPnPDeviceInfo *info,
