@@ -3,20 +3,8 @@
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -28,6 +16,20 @@
 #include "gvalue-util.h"
 #include "xml-util.h"
 
+struct _ActionArgument {
+        char *name;
+        GValue value;
+};
+typedef struct _ActionArgument ActionArgument;
+
+void
+action_argument_free (ActionArgument* arg)
+{
+        g_free (arg->name);
+        g_value_unset (&arg->value);
+        g_free (arg);
+}
+
 /* Reads a value into the parameter name and initialised GValue pair
  * from @response */
 static void
@@ -191,11 +193,6 @@ check_action_response (G_GNUC_UNUSED GUPnPServiceProxy *proxy,
 }
 
 
-/* GDestroyNotify for GHashTable holding GValues.
- */
-G_GNUC_INTERNAL void
-_value_free (gpointer data);
-
 GUPnPServiceProxyAction *
 gupnp_service_proxy_action_new_internal (const char *action) {
         GUPnPServiceProxyAction *ret;
@@ -204,6 +201,8 @@ gupnp_service_proxy_action_new_internal (const char *action) {
 
         ret = g_atomic_rc_box_new0 (GUPnPServiceProxyAction);
         ret->name = g_strdup (action);
+        ret->args = g_ptr_array_new_with_free_func((GDestroyNotify)action_argument_free);
+        ret->arg_map = g_hash_table_new (g_str_hash, g_str_equal);
 
         return ret;
 }
@@ -236,6 +235,8 @@ action_dispose (GUPnPServiceProxyAction *action)
                 g_string_free (action->msg_str, TRUE);
                 action->msg_str = NULL;
         }
+        g_hash_table_destroy (action->arg_map);
+        g_ptr_array_unref (action->args);
         g_free (action->name);
 }
 
@@ -254,14 +255,13 @@ G_DEFINE_BOXED_TYPE (GUPnPServiceProxyAction,
 
 /* Writes a parameter name and GValue pair to @msg */
 static void
-write_in_parameter (const char *arg_name,
-                    GValue     *value,
+write_in_parameter (ActionArgument *arg,
                     GString    *msg_str)
 {
         /* Write parameter pair */
-        xml_util_start_element (msg_str, arg_name);
-        gvalue_util_value_append_to_xml_string (value, msg_str);
-        xml_util_end_element (msg_str, arg_name);
+        xml_util_start_element (msg_str, arg->name);
+        gvalue_util_value_append_to_xml_string (&arg->value, msg_str);
+        xml_util_end_element (msg_str, arg->name);
 }
 
 static void
@@ -277,26 +277,93 @@ write_footer (GUPnPServiceProxyAction *action)
                          "</s:Envelope>");
 }
 
+/**
+ * gupnp_service_proxy_action_new:
+ * @action: The name of a remote action to call
+ * @...: tuples of in parameter name, in parameter type, and in parameter
+ * value, terminated with %NULL
+ *
+ * Prepares action @action with parameters @Varargs to be sent off to
+ * a remote service later with gupnp_service_proxy_call_action() or
+ * gupnp_service_proxy_call_action_async().
+ *
+ * After the action call has finished, the results of the call may be
+ * retrived from the #GUPnPServiceProxyAction by using
+ * gupnp_service_proxy_action_get_result(),
+ * gupnp_service_proxy_action_get_result_list() or
+ * gupnp_service_proxy_action_get_result_hash()
+ *
+ * <informalexample>
+ * <programlisting>
+ * GUPnPServiceProxyAction *action =
+ *         gupnp_service_proxy_action_new ("GetVolume",
+ *                                         // Parameters
+ *                                         "InstanceID", G_TYPE_INT, 0,
+ *                                         "Channel", G_TYPE_STRING, "Master",
+ *                                         NULL);
+ *
+ * GError *error = NULL;
+ * gupnp_service_proxy_call_action (proxy, action, NULL, &error);
+ * if (error != NULL) {
+ *         g_warning ("Failed to call GetVolume: %s", error->message);
+ *         g_clear_error (&error);
+ *
+ *         return;
+ * }
+ *
+ * guint16 volume = 0;
+ * if (!gupnp_service_proxy_action_get_result (action,
+ *                                             &error,
+ *                                             "CurrentVolume", G_TYPE_UINT, &volume,
+ *                                             NULL)) {
+ *         g_message ("Current Volume: %u", volume);
+ * }
+ *
+ * gupnp_service_proxy_action_unref (action);
+ * </programlisting>
+ * </informalexample>
+ *
+ * Returns: A newly created #GUPnPServiceProxyAction
+ */
 GUPnPServiceProxyAction *
 gupnp_service_proxy_action_new (const char *action,
                                 ...)
 {
-        GList *in_names = NULL;
-        GList *in_values = NULL;
         GUPnPServiceProxyAction *result = NULL;
         va_list var_args;
 
         g_return_val_if_fail (action != NULL, NULL);
 
         va_start (var_args, action);
-        VAR_ARGS_TO_IN_LIST (var_args, in_names, in_values);
-        va_end (var_args);
+        const char *arg_name = va_arg (var_args, const char *);
 
-        result = gupnp_service_proxy_action_new_from_list (action,
-                                                           in_names,
-                                                           in_values);;
-        g_list_free_full (in_names, g_free);
-        g_list_free_full (in_values, gvalue_free);
+        result = gupnp_service_proxy_action_new_internal (action);
+
+        gint position = 0;
+        while (arg_name != NULL) {
+                ActionArgument *arg = g_new0 (ActionArgument, 1);
+                arg->name = g_strdup (arg_name);
+
+                GType type = va_arg (var_args, GType);
+                char *error = NULL;
+
+                G_VALUE_COLLECT_INIT (&arg->value, type, var_args, 0, &error);
+                if (error == NULL) {
+                        g_hash_table_insert (result->arg_map,
+                                             arg->name,
+                                             GUINT_TO_POINTER (position));
+                        g_ptr_array_add (result->args, arg);
+                } else {
+                        g_warning (
+                                "Failed to collect value of type %s for %s: %s",
+                                g_type_name (type),
+                                arg_name,
+                                error);
+                        g_free (error);
+                }
+                arg_name = va_arg (var_args, const char *);
+        }
+        va_end (var_args);
 
         return result;
 }
@@ -308,6 +375,46 @@ gupnp_service_proxy_action_new (const char *action,
  * names (as strings)
  * @in_values: (element-type GValue) (transfer none): #GList of values (as
  * #GValue) that line up with @in_names
+ *
+ * Prepares action @action with parameters @in_names and @in_values to be
+ * sent off to a remote service later with gupnp_service_proxy_call_action() or
+ * gupnp_service_proxy_call_action_async(). This is mainly useful for language
+ * bindings.
+ *
+ * After the action call has finished, the results of the call may be
+ * retrived from the #GUPnPServiceProxyAction by using
+ * gupnp_service_proxy_action_get_result(),
+ * gupnp_service_proxy_action_get_result_list() or
+ * gupnp_service_proxy_action_get_result_hash()
+ * <informalexample>
+ * <programlisting>
+ * GList *in_args = NULL;
+ * in_args = g_list_append (in_args, "InstanceID");
+ * in_args = g_list_append (in_args, "Unit");
+ * in_args = g_list_append (in_args, "Target");
+ *
+ * GValue instance = G_VALUE_INIT;
+ * g_value_set_int (&instance, 0);
+ * GValue unit = G_VALUE_INIT;
+ * g_value_set_static_string (&unit, "ABS_TIME");
+ * GValue target = G_VALUE_INIT;
+ * g_value_set_static_string (&target, "00:00:00.000");
+ *
+ * GList *in_values = NULL;
+ * in_values = g_list_append (in_values, &instance);
+ * in_values = g_list_append (in_values, &unit);
+ * in_values = g_list_append (in_values, &target);
+ *
+ * GUPnPServiceProxyAction *action =
+ *         gunp_service_proxy_action_new_from_list ("Seek", in_args, in_values);
+ *
+ * GError *error = NULL;
+ * gupnp_service_proxy_call_action_async (proxy, action, NULL, on_action_finished, NULL);
+ * gupnp_service_proxy_action_unref (action);
+ * </programlisting>
+ * </informalexample>
+ *
+ * Returns: A newly created #GUPnPServiceProxyAction
  */
 GUPnPServiceProxyAction *
 gupnp_service_proxy_action_new_from_list (const char *action_name,
@@ -316,34 +423,62 @@ gupnp_service_proxy_action_new_from_list (const char *action_name,
 {
         GUPnPServiceProxyAction *action;
         GList *names, *values;
+        guint position;
 
         action = gupnp_service_proxy_action_new_internal (action_name);
+
+        /* Arguments */
+        for (names = in_names, values = in_values, position = 0;
+             names && values;
+             names = names->next, values = values->next, position++) {
+                GValue *val = values->data;
+
+                ActionArgument *arg = g_new0 (ActionArgument, 1);
+                arg->name = g_strdup (names->data);
+                g_value_init (&arg->value, G_VALUE_TYPE (val));
+                g_value_copy (val, &arg->value);
+                g_hash_table_insert (action->arg_map,
+                                     arg->name,
+                                     GUINT_TO_POINTER (position));
+                g_ptr_array_add (action->args, arg);
+        }
+
+        return action;
+}
+
+void
+gupnp_service_proxy_action_serialize (GUPnPServiceProxyAction *action,
+                                      const char *service_type)
+{
+        if (action->msg_str != NULL) {
+                g_string_free (action->msg_str, TRUE);
+        }
         action->msg_str = xml_util_new_string ();
 
         g_string_append (action->msg_str,
                          "<?xml version=\"1.0\"?>"
                          "<s:Envelope xmlns:s="
-                                "\"http://schemas.xmlsoap.org/soap/envelope/\" "
-                          "s:encodingStyle="
-                                "\"http://schemas.xmlsoap.org/soap/encoding/\">"
+                         "\"http://schemas.xmlsoap.org/soap/envelope/\" "
+                         "s:encodingStyle="
+                         "\"http://schemas.xmlsoap.org/soap/encoding/\">"
                          "<s:Body>");
         action->header_pos = action->msg_str->len;
 
-        /* Arguments */
-        for (names = in_names, values = in_values;
-             names && values;
-             names=names->next, values = values->next) {
-                GValue* val = values->data;
+        g_ptr_array_foreach (action->args,
+                             (GFunc) write_in_parameter,
+                             action->msg_str);
 
-                write_in_parameter (names->data,
-                                    val,
-                                    action->msg_str);
-        }
-
-        /* Finish and send off */
         write_footer (action);
 
-        return action;
+        g_string_insert (action->msg_str, action->header_pos, "<u:");
+        action->header_pos += strlen ("<u:");
+        g_string_insert (action->msg_str, action->header_pos, action->name);
+        action->header_pos += strlen (action->name);
+        g_string_insert (action->msg_str, action->header_pos, " xmlns:u=\"");
+        action->header_pos += strlen (" xmlns:u=\"");
+        g_string_insert (action->msg_str, action->header_pos, service_type);
+        action->header_pos += strlen (service_type);
+        g_string_insert (action->msg_str, action->header_pos, "\">");
 }
 
 /**
@@ -355,15 +490,64 @@ gupnp_service_proxy_action_new_from_list (const char *action_name,
  * that line up with @out_names
  * @out_values: (element-type GValue) (transfer full) (out): #GList of values
  * (as #GValue) that line up with @out_names and @out_types.
- * @error: The location where to store any error, or %NULL
+ * @error:(inout)(optional)(nullable): The location where to store any error, or %NULL
  *
  * A variant of gupnp_service_proxy_action_get_result() that takes lists of
  * out-parameter names, types and place-holders for values. The returned list
  * in @out_values must be freed using #g_list_free and each element in it using
  * #g_value_unset and #g_free.
+ * <informalexample>
+ * <programlisting>
+ * void on_action_finished(GObject *object, GAsyncResult *res, gpointer user_data)
+ * {
+ *     GUPnPServiceProxyAction *action;
+ *     GError *error;
+ *
+ *     action = gupnp_service_proxy_call_action_finish (GUPNP_SERVICE_PROXY (object),
+ *                                                      res,
+ *                                                      &error);
+ *
+ *     if (error != NULL) {
+ *              g_print ("Call failed: %s", error->message);
+ *              g_clear_error (&error);
+ *              return;
+ *     }
+ *
+ *     GList *out_args = NULL;
+ *     out_args = g_list_append (out_args, "PlayMode");
+ *     out_args = g_list_append (out_args, "RecQualityMode");
+ *     GList *out_types = NULL;
+ *     out_types = g_list_append (out_types, GSIZE_TO_POINTER (G_TYPE_STRING));
+ *     out_types = g_list_append (out_types, GSIZE_TO_POINTER (G_TYPE_STRING));
+ *     GList *out_values = NULL;
+ *
+ *     if (!gupnp_service_proxy_action_get_result_list (action,
+ *                                                      out_args,
+ *                                                      out_types,
+ *                                                      &out_values,
+ *                                                      &error)) {
+ *              g_print ("Getting results failed: %s", error->message);
+ *              g_clear_error (&error);
+ *              return;
+ *     }
+ *
+ *     GList *iter = out_values;
+ *     while (iter != NULL) {
+ *         GValue *value = iter->data;
+ *         g_print ("Result: %s\n", g_value_get_string (value));
+ *         g_value_unset (value);
+ *         g_free (value);
+ *         iter = g_list_remove_link (iter, iter);
+ *     }
+ *     g_list_free (out_values);
+ * }
+ * </programlisting>
+ * </informalexample>
  *
  * Return value : %TRUE on success.
  *
+ * Since: 1.2.0
+ *
  **/
 gboolean
 gupnp_service_proxy_action_get_result_list (GUPnPServiceProxyAction *action,
@@ -426,13 +610,59 @@ gupnp_service_proxy_action_get_result_list (GUPnPServiceProxyAction *action,
  * @action: A #GUPnPServiceProxyAction handle
  * @out_hash: (element-type utf8 GValue) (inout) (transfer none): A #GHashTable of
  * out parameter name and initialised #GValue pairs
- * @error: The location where to store any error, or %NULL
+ * @error:(inout)(optional)(nullable): The location where to store any error, or %NULL
  *
  * See gupnp_service_proxy_action_get_result(); this version takes a #GHashTable for
  * runtime generated parameter lists.
  *
+ * The @out_hash needs to be pre-initialized with key value pairs denoting the argument
+ * to retrieve and an empty #GValue initialized to hold the wanted type with g_value_init().
+ *
+ * <informalexample>
+ * <programlisting>
+ * void on_action_finished(GObject *object, GAsyncResult *res, gpointer user_data)
+ * {
+ *     GUPnPServiceProxyAction *action;
+ *     GError *error;
+ *
+ *     action = gupnp_service_proxy_call_action_finish (GUPNP_SERVICE_PROXY (object),
+ *                                                      res,
+ *                                                      &error);
+ *
+ *     if (error != NULL) {
+ *              g_print ("Call failed: %s", error->message);
+ *              g_clear_error (&error);
+ *              return;
+ *     }
+ *
+ *     GValue play_mode = G_VALUE_INIT;
+ *     g_value_init(&play_mode, G_TYPE_STRING);
+ *     GValue rec_quality_mode = G_VALUE_INIT;
+ *     g_value_init(&rec_quality_mode, G_TYPE_STRING);
+ *
+ *     GHashTable *out_args = g_hash_table_new (g_str_hash, g_str_equal);
+ *     g_hash_table_insert(out_args, "PlayMode", &play_mode);
+ *     g_hash_table_insert(out_args, "RecQualityMode", &rec_quality_mode);
+ *
+ *     if (!gupnp_service_proxy_action_get_result_hash (action,
+ *                                                      out_args,
+ *                                                      &error)) {
+ *              g_print ("Getting results failed: %s", error->message);
+ *              g_clear_error (&error);
+ *              return;
+ *     }
+ *
+ *     g_value_unset (&play_mode);
+ *     g_value_unset (&rec_quality_mode);
+ *
+ *     g_hash_table_unref (out_args);
+ * }
+ * </programlisting>
+ * </informalexample>
+ *
  * Return value: %TRUE on success.
  *
+ * Since: 1.2.0
  **/
 gboolean
 gupnp_service_proxy_action_get_result_hash (GUPnPServiceProxyAction *action,
@@ -472,7 +702,7 @@ gupnp_service_proxy_action_get_result_hash (GUPnPServiceProxyAction *action,
 /**
  * gupnp_service_proxy_action_get_result:
  * @action: A #GUPnPServiceProxyAction handle
- * @error: (allow-none): The location where to store any error, or %NULL
+ * @error: (inout)(optional)(nullable): The location where to store any error, or %NULL
  * @...: tuples of out parameter name, out parameter type, and out parameter
  * value location, terminated with %NULL. The out parameter values should be
  * freed after use
@@ -503,7 +733,7 @@ gupnp_service_proxy_action_get_result (GUPnPServiceProxyAction *action,
 /**
  * gupnp_service_proxy_action_get_result_valist:
  * @action: A #GUPnPServiceProxyAction handle
- * @error: (allow-none): The location where to store any error, or %NULL
+ * @error: (inout)(optional)(nullable): The location where to store any error, or %NULL
  * @var_args: A va_list of tuples of out parameter name, out parameter type,
  * and out parameter value location. The out parameter values should be
  * freed after use
@@ -545,3 +775,49 @@ gupnp_service_proxy_action_get_result_valist (GUPnPServiceProxyAction *action,
 
         return result;
 }
+
+gboolean
+gupnp_service_proxy_action_set (GUPnPServiceProxyAction *action,
+                                const char *key,
+                                const GValue *value,
+                                GError **error)
+{
+        g_return_val_if_fail (key != NULL, FALSE);
+        g_return_val_if_fail (value != NULL, FALSE);
+        g_return_val_if_fail (error != NULL && *error == NULL, FALSE);
+        gpointer position;
+
+        if (!g_hash_table_lookup_extended (action->arg_map,
+                                           key,
+                                           NULL,
+                                           &position)) {
+                g_propagate_error (error,
+                                   g_error_new (GUPNP_SERVER_ERROR,
+                                                GUPNP_SERVER_ERROR_OTHER,
+                                                "Unknown argument: %s",
+                                                key));
+
+                return FALSE;
+        }
+
+        ActionArgument *arg =
+                g_ptr_array_index (action->args, GPOINTER_TO_UINT (position));
+
+        if (G_VALUE_TYPE (value) != G_VALUE_TYPE (&arg->value)) {
+                g_propagate_error (
+                        error,
+                        g_error_new (
+                                GUPNP_SERVER_ERROR,
+                                GUPNP_SERVER_ERROR_OTHER,
+                                "Type mismatch for %s. Expected %s, got %s",
+                                key,
+                                G_VALUE_TYPE_NAME (&arg->value),
+                                G_VALUE_TYPE_NAME (value)));
+
+                return FALSE;
+        }
+
+        g_value_copy (value, &arg->value);
+
+        return TRUE;
+}
