@@ -3,20 +3,7 @@
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifndef GUPNP_SERVICE_PROXY_H
@@ -89,20 +76,20 @@ typedef void (* GUPnPServiceProxyNotifyCallback) (GUPnPServiceProxy *proxy,
                                                   GValue            *value,
                                                   gpointer           user_data);
 
-gboolean
+G_DEPRECATED gboolean
 gupnp_service_proxy_send_action    (GUPnPServiceProxy              *proxy,
                                     const char                     *action,
                                     GError                        **error,
                                     ...) G_GNUC_NULL_TERMINATED;
 
-gboolean
+G_DEPRECATED gboolean
 gupnp_service_proxy_send_action_valist
                                    (GUPnPServiceProxy              *proxy,
                                     const char                     *action,
                                     GError                        **error,
                                     va_list                         var_args);
 
-gboolean
+G_DEPRECATED gboolean
 gupnp_service_proxy_send_action_list (GUPnPServiceProxy *proxy,
                                       const char        *action,
                                       GList             *in_names,
@@ -111,14 +98,15 @@ gupnp_service_proxy_send_action_list (GUPnPServiceProxy *proxy,
                                       GList             *out_types,
                                       GList            **out_values,
                                       GError           **error);
-GUPnPServiceProxyAction *
+
+G_DEPRECATED GUPnPServiceProxyAction *
 gupnp_service_proxy_begin_action   (GUPnPServiceProxy              *proxy,
                                     const char                     *action,
                                     GUPnPServiceProxyActionCallback callback,
                                     gpointer                        user_data,
                                     ...) G_GNUC_NULL_TERMINATED;
 
-GUPnPServiceProxyAction *
+G_DEPRECATED GUPnPServiceProxyAction *
 gupnp_service_proxy_begin_action_valist
                                    (GUPnPServiceProxy              *proxy,
                                     const char                     *action,
@@ -126,7 +114,7 @@ gupnp_service_proxy_begin_action_valist
                                     gpointer                        user_data,
                                     va_list                         var_args);
 
-GUPnPServiceProxyAction *
+G_DEPRECATED GUPnPServiceProxyAction *
 gupnp_service_proxy_begin_action_list
                                    (GUPnPServiceProxy              *proxy,
                                     const char                     *action,
@@ -135,20 +123,20 @@ gupnp_service_proxy_begin_action_list
                                     GUPnPServiceProxyActionCallback callback,
                                     gpointer                        user_data);
 
-gboolean
+G_DEPRECATED gboolean
 gupnp_service_proxy_end_action     (GUPnPServiceProxy              *proxy,
                                     GUPnPServiceProxyAction        *action,
                                     GError                        **error,
                                     ...) G_GNUC_NULL_TERMINATED;
 
-gboolean
+G_DEPRECATED gboolean
 gupnp_service_proxy_end_action_valist
                                    (GUPnPServiceProxy              *proxy,
                                     GUPnPServiceProxyAction        *action,
                                     GError                        **error,
                                     va_list                         var_args);
 
-gboolean
+G_DEPRECATED gboolean
 gupnp_service_proxy_end_action_list
                                   (GUPnPServiceProxy       *proxy,
                                    GUPnPServiceProxyAction *action,
@@ -157,14 +145,14 @@ gupnp_service_proxy_end_action_list
                                    GList                  **out_values,
                                    GError                  **error);
 
-gboolean
+G_DEPRECATED gboolean
 gupnp_service_proxy_end_action_hash
                                    (GUPnPServiceProxy              *proxy,
                                     GUPnPServiceProxyAction        *action,
                                     GHashTable                     *hash,
                                     GError                        **error);
 
-void
+G_DEPRECATED void
 gupnp_service_proxy_cancel_action  (GUPnPServiceProxy              *proxy,
                                     GUPnPServiceProxyAction        *action);
 
@@ -208,6 +196,61 @@ gupnp_service_proxy_set_subscribed (GUPnPServiceProxy              *proxy,
 gboolean
 gupnp_service_proxy_get_subscribed (GUPnPServiceProxy              *proxy);
 
+/* New action API */
+
+GUPnPServiceProxyAction *
+gupnp_service_proxy_action_new (const char *action,
+                                ...) G_GNUC_NULL_TERMINATED;
+
+GUPnPServiceProxyAction *
+gupnp_service_proxy_action_new_from_list (const char *action,
+                                          GList      *in_names,
+                                          GList      *in_values);
+
+GUPnPServiceProxyAction *
+gupnp_service_proxy_action_ref (GUPnPServiceProxyAction *action);
+
+void
+gupnp_service_proxy_action_unref (GUPnPServiceProxyAction *action);
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GUPnPServiceProxyAction,
+                               gupnp_service_proxy_action_unref)
+
+gboolean
+gupnp_service_proxy_action_get_result (GUPnPServiceProxyAction *action,
+                                       GError                 **error,
+                                       ...) G_GNUC_NULL_TERMINATED;
+
+gboolean
+gupnp_service_proxy_action_get_result_list (GUPnPServiceProxyAction *action,
+                                            GList                   *out_names,
+                                            GList                   *out_types,
+                                            GList                  **out_values,
+                                            GError                 **error);
+gboolean
+gupnp_service_proxy_action_get_result_hash (GUPnPServiceProxyAction *action,
+                                            GHashTable              *out_hash,
+                                            GError                 **error);
+
+
+void
+gupnp_service_proxy_call_action_async (GUPnPServiceProxy       *proxy,
+                                       GUPnPServiceProxyAction *action,
+                                       GCancellable            *cancellable,
+                                       GAsyncReadyCallback     callback,
+                                       gpointer                user_data);
+
+
+GUPnPServiceProxyAction *
+gupnp_service_proxy_call_action_finish (GUPnPServiceProxy *proxy,
+                                        GAsyncResult      *result,
+                                        GError           **error);
+
+GUPnPServiceProxyAction *
+gupnp_service_proxy_call_action (GUPnPServiceProxy       *proxy,
+                                 GUPnPServiceProxyAction *action,
+                                 GCancellable            *cancellable,
+                                 GError                 **error);
 
 G_END_DECLS
 
